/* TM1637_clock2

   Ébresztőóra egy DS3231 Real Time óra és egy
   TM1637 négyjegyű 7-szegmens kijelző felhasználásával.
   Az óra beállítása két nyomógombbal történik:
    - a G1 gomb lépteti az óra állapotát (0: kijelzés,
      1: óra beállítás, 2: perc beállítás, 3: ébresztési óra
      beállítás, 4: ébresztési perc beállítása, 5: ébresztés ki/be
      kapcsolása)
    - a G2 gomb a kiválasztott paramétert lépteti

   Ébresztéskor a D13-as kimenetet 10 másodpercig magas állapotba
   kapcsoljuk.

   A programhoz felhasználtuk az Avishay Orpaz által
   írt TM1637Display programkönyvtárat:
   https://github.com/avishorp/TM1637
   valamint Andrew Wickert DS3231 programkönyvtárát:
   https://github.com/NorthernWidget/DS3231
*/
#include <TM1637Display.h>
#include <DS3231.h>
#include <Wire.h>

#define CLKPIN  9                     // TM1637 CLK láb D9-re
#define DIOPIN  8                     // TM1637 DIO láb D8-ra
#define G1      7                     // Button 1
#define G2     10                     // Button 2 
#define SQW     2                     // Megszakítás
#define BUZZER 13                     // Alarm kimenet

DS3231 Clock;                         // Az RTC példányosítása
TM1637Display display(CLKPIN, DIOPIN); // 4-számjegyű kijelző példányosítása

bool h12, PM;                         // 12/24 mód és AM/PM jelzőbitek
int hh = 0;                           // órák száma
int mm = 0;                           // percek száma
int dp = 0;                           // Kettőspont kijelzés (0 vagy 64)
uint32_t g1low = 0;                   // G1 alacsony állapot számlálója
uint32_t g1high = 0;                  // G1 magas állapot számlálója


byte state = 0;
byte A2Day;
byte A2Hour;
byte A2Minute;
byte AlarmBits;
bool A2Dy;
bool A2h12;
bool A2PM;
bool alarm;

void setup() {
  pinMode(G1, INPUT_PULLUP);
  pinMode(G2, INPUT_PULLUP);
  pinMode(SQW, INPUT_PULLUP);
  pinMode(BUZZER, OUTPUT);
  digitalWrite(BUZZER, LOW);
  Wire.begin();                       // Az I2C illesztő inicializálása
  display.setBrightness(0x0F);        // a kijelző bekapcsolása max. fényerőn
  Clock.enableOscillator(true, 0, 0);
}

void loop() {
  uint16_t d12, d34;
  uint8_t data[] = { 0, 0, 0, 0 };

  /*----------------------------------------------
     A G1 gomb lenyomásának vizsgálata
     Érvényes lenyomás esetén állapotot váltunk
    ----------------------------------------------*/
  if ( digitalRead(G1) == LOW) {
    g1low++;
    if ((g1low > 5) && (g1high > 5)) {
      g1high = 0;
      g1low = 0;
      state = (state + 1 ) % 6;
    }
  } else {
    g1high++;
    g1low = 0;
  }
  /*----------------------------------------------
     Az állapotgép kezelése: state > 0 esetén
     G2 lenyomásakor az aktuális paramétert
     léptetjük és kiíratjuk
    ----------------------------------------------*/
  switch (state) {
    case 0:                               // Kijelzés mód
      //-- Ébresztés, ha itt az idő -------------
      if (Clock.checkIfAlarm(2) ) {
        digitalWrite(BUZZER, HIGH);       // Alarm aktiválás
      }
      //-- Ébresztés megszüntetése 10 mp után ---
      if (Clock.getSecond() > 10) {
        digitalWrite(BUZZER, LOW);        // Alarm vége
      }
      //-- Az 1 Hz-es jel figyelése -------------
      dp = digitalRead(SQW) ? 64 : 0;
      break;                       // nincs mit tenni

    case 1:                               // setHH mód
      hh = Clock.getHour(h12, PM);
      if ( digitalRead(G2) == LOW) {      // Ha a G2 gomb le van nyomva...
        hh = (hh + 1) % 24;               // Az órák léptetése
        Clock.setHour(hh);                // Az új érték eltárolása
        delay(100);
      }
      d12 = 0x7676;                       // HH kijelzés
      d34 = hh;                           // óra kijelzés
      break;

    case 2:                               // setMM mód
      mm = Clock.getMinute();
      if ( digitalRead(G2) == LOW) {      // Ha a G2 gomb le van nyomva...
        mm = (mm + 1) % 60;               // A percek léptetése
        Clock.setMinute(mm);              // Percek eltárolása
        Clock.setSecond(0);               // Másodpercek nullázása
        delay(100);
      }
      d12 = 0x7373;                       // PP kijelzés
      d34 = mm;                           // percek kijelzése
      break;

    case 3:                               // setAH mód
      Clock.getA2Time(A2Day, A2Hour, A2Minute, AlarmBits, A2Dy, A2h12, A2PM);
      if ( digitalRead(G2) == LOW) {      // Ha a G2 gomb le van nyomva...
        A2Day = Clock.getDoW();           // Az ébresztés ma történik, vagy holnap
        A2Dy = true;                      // A hét napja egyezést figyelje
        AlarmBits = 8;                    // Alarm when DoW, hour, min match
        A2Hour = (A2Hour + 1) % 24;       // Az ébresztési óra léptetése
        if (hh > A2Hour) {
          A2Day = (A2Day % 7) + 1;        // Az ébresztés csak holnap esedékes
        }
        Clock.setA2Time(A2Day, A2Hour, A2Minute, AlarmBits, A2Dy, A2h12, A2PM);
        delay(100);
      }
      d12 = 0x7776;                       // AH kijelzése
      d34 = A2Hour;                       // Ébresztési óra kijelzése
      break;

    case 4:                               // setAM mód
      Clock.getA2Time(A2Day, A2Hour, A2Minute, AlarmBits, A2Dy, A2h12, A2PM);
      if ( digitalRead(G2) == LOW) {      // Ha a G2 gomb le van nyomva...
        A2Dy = true;
        AlarmBits = 8;                    // Alarm when DoW, hour, min match
        A2Minute = (A2Minute + 1) % 60;   // Az ébresztési perc léptetése
        if (hh == A2Hour) {
          A2Day = Clock.getDoW();
          if (mm >= A2Minute) {
            A2Day = (A2Day % 7) + 1; // Az ébresztés holnap esedékes
          }
        }
        Clock.setA2Time(A2Day, A2Hour, A2Minute, AlarmBits, A2Dy, A2h12, A2PM);
        delay(100);
      }
      d12 = 0x7773;                       // AP
      d34 = A2Minute;
      break;

    case 5:                               // setAL mód
      alarm = Clock.checkAlarmEnabled(2); // 1: ha A2 alarm engedélyezve van
      if ( digitalRead(G2) == LOW) {      // Ha a G2 gomb le van nyomva...
        alarm = !alarm;                   // Toggle A2 alarm status
        if (alarm) Clock.turnOnAlarm(2);  // Ébresztés bekapcsolása
        else Clock.turnOffAlarm(2);       // Ébresztés kikapcsolása
        Clock.enableOscillator(true,0,0); // 1Hz újra beállítása
        delay(200);        
      }
      d12 = 0x7738;                       // AL
      d34 = alarm;
      break;
  }

  //-- A kijelzés frissítése --------------------
  if (state == 0) {
    hh = Clock.getHour(h12, PM);
    mm = Clock.getMinute();                       // Percek kiolcvasása
    display.showNumberDecEx(hh, dp, false, 2, 0); // Óra kijelzése kettősponttal
    display.showNumberDec(mm, true, 2, 2);        // percek kijelzése
  } else {
    data[0] = d12 >> 8;                           // Első karakter
    data[1] = d12 & 0xFF;
    data[2] = display.encodeDigit(d34 / 10);
    data[3] = display.encodeDigit(d34 % 10);
    display.setSegments(data);                    // szöveg kijelzés
  }
  delay(20);
}
